import json
import ntpath
import re
import os
import argparse
from pprint import pprint

def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)

parser = argparse.ArgumentParser(description='Lists racks used in a song')
parser.add_argument('setlist', help='path to the setlist to read')

args = parser.parse_args()
setlistFile = args.setlist

class SetList:
  def __init__(self, path):
    self.setlistDir = os.path.abspath(os.path.dirname(path))
    self.songs = []
    with open(path) as data_file:
      self.data = json.load(data_file)

      songsList = self.data.get("songs")
      for songObj in songsList:
        if songObj.get("kind") == "song":
          songFile = songObj.get("songFile")
          songFileRel = songObj.get("songFileRelative")
          songFileRel = songFileRel.replace('\\', '/')
          songFileAbs = os.path.normpath(os.path.normcase(os.path.join(self.setlistDir, songFileRel)))

          self.songs.append(Song(songFileAbs))

  def getSongs(self):
    return self.songs


class Song:
  def __init__(self, path):
    self.path = path
    self.songDir = os.path.abspath(os.path.dirname(path))
    self.name = re.sub('\.cantabileSong$', '', path_leaf(path))

    with open(path) as data_file:
      self.data = json.load(data_file)

  class SongState:
    def __init__(self, song, stateIndex, obj):
      self.song = song
      self.index = stateIndex
      self.obj = obj

  def songStates(self):
    states = []
    statesObj = self.obj.get("states")
    for stateIndex in statesObj.keys():
      stateObj = statesObj.get(stateIndex)
      states.append(SongState(this, stateIndex, stateObj))
    return states

  def getRackInstances(self):
    rackInstances = []

    for obj in self.data["objects"]:
      if isinstance(obj, dict):
        if obj.get("kind") == "RackHost":
          embeddedRack = obj.get("embeddedRack")
          if not embeddedRack:
            rackFileRel = obj.get("RackFileRelative")
            if rackFileRel:
              rackFileRel = rackFileRel.replace('\\', '/')
              rackFileAbs = os.path.normpath(os.path.normcase(os.path.join(self.songDir, rackFileRel)))
              rackInstances.append(Rack(rackFileAbs))
    return rackInstances

  def getEmbeddedRacks(self):
    embeddedRacks = []

    for obj in self.data["objects"]:
      if isinstance(obj, dict):
        if obj.get("kind") == "RackHost":
          embeddedRack = obj.get("embeddedRack")
          if embeddedRack:
            embeddedRacks.append(obj.get("name"))

    return embeddedRacks

  def getPlugins(self):
    plugins = []
    objects = self.data.get("objects")
    for obj in objects:
      if obj.get("kind") == "Plugin":
        path = obj.get("pluginPath")
        plugins.append(Plugin(path))
    return plugins

# TODO
# Make Rack the base class and have two inherited classes,
# ExternalRack and EmbeddedRack
#
# Inheritance example: http://www.jesshamrick.com/2011/05/18/an-introduction-to-classes-and-inheritance-in-python/

class Rack:
  def __init__(self, path):
    self.name = re.sub('\.cantabileRack$', '', path_leaf(path))
    with open(path) as data_file:
      self.data = json.load(data_file)

  def rackStates(self):
    states = []
    statesObj = self.data.get("states")
    for stateIndex in statesObj.keys():
      stateObj = statesObj.get(stateIndex)
      states.append(RackState(self, stateIndex))
    return states

  def getPlugins(self):
    plugins = []
    objects = self.data.get("objects")
    for obj in objects:
      if obj.get("kind") == "Plugin":
        path = obj.get("pluginPath")
        plugins.append(Plugin(path))
    return plugins


class RackState:
  def __init__(self, rack, stateIndex, obj):
    self.rack = rack
    self.index = stateIndex
    self.obj = obj


class Plugin:
  def __init__(self, path):
    self.path = path

  def toString(self):
    return "%s" % (self.path)


print ("\n***** Racks used by each song *****")

rackToSong = {}
rackNameToRack = {}

setlist = SetList(setlistFile)
for song in setlist.getSongs():
  print (song.name)

  rackInstances = song.getRackInstances()
  for rackInstance in rackInstances:
    rackName = rackInstance.name
    rackNameToRack[rackName] = rackInstance
    print ("  %s" % (rackName))

    if not rackToSong.has_key(rackName):
      rackToSong[rackName] = []

    rackToSong[rackName].append(song.name)

  embeddedRacks = song.getEmbeddedRacks()
  for rackName in embeddedRacks:
    print ("  ==Embedded== %s" % (rackName))

print ("\n***** Songs using each rack *****")
for rackName in rackToSong.keys():
  print ("Rack: %s" % (rackName))

  for songName in rackToSong[rackName]:
    print ("  %s" % (songName))


print ("\n***** Plugins used in setlist *****")
allPluginPaths = []
for rackName in rackNameToRack.keys():
  rack = rackNameToRack[rackName]
  plugins = rack.getPlugins()
  for plugin in plugins:
    allPluginPaths.append(plugin.path)

uniquePluginPaths = sorted(set(allPluginPaths))
for path in uniquePluginPaths:
  print (path)

